@@ -34,6 +34,7 @@ from sphinx.writers.html import HTMLTranslator
 from sphinx.locale import versionlabels
 HTMLTranslator.visit_versionmodified = new_visit_versionmodified
 
+
 # Support for marking up and linking to bugs.python.org issues
 
 def issue_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
@@ -99,19 +100,8 @@ from pprint import pformat
 from docutils.io import StringOutput
 from docutils.utils import new_document
 
-try:
-    from sphinx.builders import Builder
-except ImportError:
-    # using Sphinx < 0.6, which has a different package layout
-    from sphinx.builder import Builder
-    # monkey-patch toctree directive to accept (and ignore) the :numbered: flag
-    from sphinx.directives.other import toctree_directive
-    toctree_directive.options['numbered'] = toctree_directive.options['glob']
-
-try:
-    from sphinx.writers.text import TextWriter
-except ImportError:
-    from sphinx.textwriter import TextWriter
+from sphinx.builders import Builder
+from sphinx.writers.text import TextWriter
 
 
 class PydocTopicsBuilder(Builder):
@@ -128,7 +118,9 @@ class PydocTopicsBuilder(Builder):
 
     def write(self, *ignored):
         writer = TextWriter(self)
-        for label in self.status_iterator(pydoc_topic_labels, 'building topics... '):
+        for label in self.status_iterator(pydoc_topic_labels,
+                                          'building topics... ',
+                                          length=len(pydoc_topic_labels)):
             if label not in self.env.labels:
                 self.warn('label %r not in documentation' % label)
                 continue
@@ -141,7 +133,7 @@ class PydocTopicsBuilder(Builder):
             self.topics[label] = writer.output
 
     def finish(self):
-        f = open(path.join(self.outdir, 'pydoc_topics.py'), 'w')
+        f = open(path.join(self.outdir, 'topics.py'), 'w')
         try:
             f.write('# Autogenerated by Sphinx on %s\n' % asctime())
             f.write('topics = ' + pformat(self.topics) + '\n')
