@@ -21,8 +21,8 @@ CThunkObject_dealloc(PyObject *_self)
     Py_XDECREF(self->converters);
     Py_XDECREF(self->callable);
     Py_XDECREF(self->restype);
-    if (self->pcl)
-        FreeClosure(self->pcl);
+    if (self->pcl_write)
+        ffi_closure_free(self->pcl_write);
     PyObject_GC_Del(self);
 }
 
@@ -46,7 +46,7 @@ CThunkObject_clear(PyObject *_self)
     return 0;
 }
 
-PyTypeObject CThunk_Type = {
+PyTypeObject PyCThunk_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "_ctypes.CThunkObject",
     sizeof(CThunkObject),                       /* tp_basicsize */
@@ -95,44 +95,66 @@ PrintError(char *msg, ...)
     PyErr_Print();
 }
 
+#if (PY_VERSION_HEX < 0x02070000)
+PyCodeObject *
+PyCode_NewEmpty(const char *filename, const char *funcname, int firstlineno)
+{
+    static PyObject *emptystring = NULL;
+    static PyObject *nulltuple = NULL;
+    PyObject *filename_ob = NULL;
+    PyObject *funcname_ob = NULL;
+    PyCodeObject *result = NULL;
+    if (emptystring == NULL) {
+        emptystring = PyString_FromString("");
+        if (emptystring == NULL)
+            goto failed;
+    }
+    if (nulltuple == NULL) {
+        nulltuple = PyTuple_New(0);
+        if (nulltuple == NULL)
+            goto failed;
+    }
+    funcname_ob = PyString_FromString(funcname);
+    if (funcname_ob == NULL)
+        goto failed;
+    filename_ob = PyString_FromString(filename);
+    if (filename_ob == NULL)
+        goto failed;
+
+    result = PyCode_New(0,                      /* argcount */
+                0,                              /* nlocals */
+                0,                              /* stacksize */
+                0,                              /* flags */
+                emptystring,                    /* code */
+                nulltuple,                      /* consts */
+                nulltuple,                      /* names */
+                nulltuple,                      /* varnames */
+                nulltuple,                      /* freevars */
+                nulltuple,                      /* cellvars */
+                filename_ob,                    /* filename */
+                funcname_ob,                    /* name */
+                firstlineno,                    /* firstlineno */
+                emptystring                     /* lnotab */
+                );
+
+failed:
+    Py_XDECREF(funcname_ob);
+    Py_XDECREF(filename_ob);
+    return result;
+}
+#endif
+
 
 /* after code that pyrex generates */
-void _AddTraceback(char *funcname, char *filename, int lineno)
+void _ctypes_add_traceback(char *funcname, char *filename, int lineno)
 {
-    PyObject *py_srcfile = 0;
-    PyObject *py_funcname = 0;
     PyObject *py_globals = 0;
-    PyObject *empty_tuple = 0;
-    PyObject *empty_string = 0;
     PyCodeObject *py_code = 0;
     PyFrameObject *py_frame = 0;
 
-    py_srcfile = PyString_FromString(filename);
-    if (!py_srcfile) goto bad;
-    py_funcname = PyString_FromString(funcname);
-    if (!py_funcname) goto bad;
     py_globals = PyDict_New();
     if (!py_globals) goto bad;
-    empty_tuple = PyTuple_New(0);
-    if (!empty_tuple) goto bad;
-    empty_string = PyString_FromString("");
-    if (!empty_string) goto bad;
-    py_code = PyCode_New(
-        0,            /*int argcount,*/
-        0,            /*int nlocals,*/
-        0,            /*int stacksize,*/
-        0,            /*int flags,*/
-        empty_string, /*PyObject *code,*/
-        empty_tuple,  /*PyObject *consts,*/
-        empty_tuple,  /*PyObject *names,*/
-        empty_tuple,  /*PyObject *varnames,*/
-        empty_tuple,  /*PyObject *freevars,*/
-        empty_tuple,  /*PyObject *cellvars,*/
-        py_srcfile,   /*PyObject *filename,*/
-        py_funcname,  /*PyObject *name,*/
-        lineno,   /*int firstlineno,*/
-        empty_string  /*PyObject *lnotab*/
-        );
+    py_code = PyCode_NewEmpty(filename, funcname, lineno);
     if (!py_code) goto bad;
     py_frame = PyFrame_New(
         PyThreadState_Get(), /*PyThreadState *tstate,*/
@@ -145,10 +167,6 @@ void _AddTraceback(char *funcname, char *filename, int lineno)
     PyTraceBack_Here(py_frame);
   bad:
     Py_XDECREF(py_globals);
-    Py_XDECREF(py_srcfile);
-    Py_XDECREF(py_funcname);
-    Py_XDECREF(empty_tuple);
-    Py_XDECREF(empty_string);
     Py_XDECREF(py_code);
     Py_XDECREF(py_frame);
 }
@@ -227,7 +245,7 @@ static void _CallPythonObject(void *mem,
             goto Done;
         }
 
-        if (dict && dict->getfunc && !IsSimpleSubType(cnv)) {
+        if (dict && dict->getfunc && !_ctypes_simple_instance(cnv)) {
             PyObject *v = dict->getfunc(*pArgs, dict->size);
             if (!v) {
                 PrintError("create argument %d:\n", i);
@@ -238,10 +256,10 @@ static void _CallPythonObject(void *mem,
             /* XXX XXX XX
                We have the problem that c_byte or c_short have dict->size of
                1 resp. 4, but these parameters are pushed as sizeof(int) bytes.
-               BTW, the same problem occurrs when they are pushed as parameters
+               BTW, the same problem occurs when they are pushed as parameters
             */
         } else if (dict) {
-            /* Hm, shouldn't we use CData_AtAddress() or something like that instead? */
+            /* Hm, shouldn't we use PyCData_AtAddress() or something like that instead? */
             CDataObject *obj = (CDataObject *)PyObject_CallFunctionObjArgs(cnv, NULL);
             if (!obj) {
                 PrintError("create argument %d:\n", i);
@@ -272,10 +290,10 @@ static void _CallPythonObject(void *mem,
     }
 
 #define CHECK(what, x) \
-if (x == NULL) _AddTraceback(what, "_ctypes/callbacks.c", __LINE__ - 1), PyErr_Print()
+if (x == NULL) _ctypes_add_traceback(what, "_ctypes/callbacks.c", __LINE__ - 1), PyErr_Print()
 
     if (flags & (FUNCFLAG_USE_ERRNO | FUNCFLAG_USE_LASTERROR)) {
-        error_object = get_error_object(&space);
+        error_object = _ctypes_get_errobj(&space);
         if (error_object == NULL)
             goto Done;
         if (flags & FUNCFLAG_USE_ERRNO) {
@@ -332,7 +350,7 @@ if (x == NULL) _AddTraceback(what, "_ctypes/callbacks.c", __LINE__ - 1), PyErr_P
             PyErr_WriteUnraisable(callable);
         else if (keep == Py_None) /* Nothing to keep */
             Py_DECREF(keep);
-        else if (setfunc != getentry("O")->setfunc) {
+        else if (setfunc != _ctypes_get_fielddesc("O")->setfunc) {
             if (-1 == PyErr_Warn(PyExc_RuntimeWarning,
                                  "memory leak in callback function."))
                 PyErr_WriteUnraisable(callable);
@@ -367,13 +385,14 @@ static CThunkObject* CThunkObject_new(Py_ssize_t nArgs)
     CThunkObject *p;
     int i;
 
-    p = PyObject_GC_NewVar(CThunkObject, &CThunk_Type, nArgs);
+    p = PyObject_GC_NewVar(CThunkObject, &PyCThunk_Type, nArgs);
     if (p == NULL) {
         PyErr_NoMemory();
         return NULL;
     }
 
-    p->pcl = NULL;
+    p->pcl_exec = NULL;
+    p->pcl_write = NULL;
     memset(&p->cif, 0, sizeof(p->cif));
     p->converters = NULL;
     p->callable = NULL;
@@ -386,7 +405,7 @@ static CThunkObject* CThunkObject_new(Py_ssize_t nArgs)
     return p;
 }
 
-CThunkObject *AllocFunctionCallback(PyObject *callable,
+CThunkObject *_ctypes_alloc_callback(PyObject *callable,
                                     PyObject *converters,
                                     PyObject *restype,
                                     int flags)
@@ -403,8 +422,9 @@ CThunkObject *AllocFunctionCallback(PyObject *callable,
 
     assert(CThunk_CheckExact(p));
 
-    p->pcl = MallocClosure();
-    if (p->pcl == NULL) {
+    p->pcl_write = ffi_closure_alloc(sizeof(ffi_closure),
+				     &p->pcl_exec);
+    if (p->pcl_write == NULL) {
         PyErr_NoMemory();
         goto error;
     }
@@ -414,7 +434,7 @@ CThunkObject *AllocFunctionCallback(PyObject *callable,
         PyObject *cnv = PySequence_GetItem(converters, i);
         if (cnv == NULL)
             goto error;
-        p->atypes[i] = GetType(cnv);
+        p->atypes[i] = _ctypes_get_ffi_type(cnv);
         Py_DECREF(cnv);
     }
     p->atypes[i] = NULL;
@@ -442,14 +462,20 @@ CThunkObject *AllocFunctionCallback(PyObject *callable,
 #endif
     result = ffi_prep_cif(&p->cif, cc,
                           Py_SAFE_DOWNCAST(nArgs, Py_ssize_t, int),
-                          GetType(restype),
+                          _ctypes_get_ffi_type(restype),
                           &p->atypes[0]);
     if (result != FFI_OK) {
         PyErr_Format(PyExc_RuntimeError,
                      "ffi_prep_cif failed with %d", result);
         goto error;
     }
-    result = ffi_prep_closure(p->pcl, &p->cif, closure_fcn, p);
+#if defined(X86_DARWIN) || defined(POWERPC_DARWIN)
+    result = ffi_prep_closure(p->pcl_write, &p->cif, closure_fcn, p);
+#else
+    result = ffi_prep_closure_loc(p->pcl_write, &p->cif, closure_fcn,
+				  p,
+				  p->pcl_exec);
+#endif
     if (result != FFI_OK) {
         PyErr_Format(PyExc_RuntimeError,
                      "ffi_prep_closure failed with %d", result);
